;
; startup bank at F000
;
; 

.include "common.inc"

; from startup.cfg
.import __RAMCODE_LOAD__
.import __RAMCODE_RUN__
.import __RAMCODE_SIZE__

; from detect_pal.s
.import getTVSystem

.segment "CODE"

load_ramcode:
    ldy #0
    ; temp_word0 = source
    lda #<__RAMCODE_LOAD__
    sta temp_word0+0
    lda #>__RAMCODE_LOAD__
    sta temp_word0+1
    ; temp_word1 = destination
    lda #<__RAMCODE_RUN__
    sta temp_word1+0
    lda #>__RAMCODE_RUN__
    sta temp_word1+1
@loop:
    ; check if finished
    lda temp_word0+1
    cmp #>(__RAMCODE_LOAD__ + __RAMCODE_SIZE__)
    bne :+
    lda temp_word0+0
    cmp #<(__RAMCODE_LOAD__ + __RAMCODE_SIZE__)
    bne :+
    rts ; finished
    :
    ; copy a byte
    lda (temp_word0),Y
    sta (temp_word1),Y
    ; increment source/destination
    inc temp_word0+0
    bne :+
        inc temp_word0+1
    :
    inc temp_word1+0
    bne :+
        inc temp_word1+1
    :
    jmp @loop


;
; POWER ON
;

power_on:
	; standard startup
	sei       ; set interrupt flag (unnecessary, unless reset is called from code)
	cld       ; disable decimal mode
	ldx #$40
	stx $4017 ; disable APU IRQ
	ldx #$ff
	txs       ; set up stack
	ldx #$00
	stx $2000 ; disable NMI
	stx $2001 ; disable render
	stx $4010 ; disable DPCM IRQ
	stx $4015 ; mute APU
	;
	bit $2002 ; clear vblank flag
	; wait for vblank
	:
		bit $2002
	bpl :-
	; save random seed
	lda prng_seed+0
	pha
	lda prng_seed+1
	pha
	; clear memory
	ldx #$00
	:
		lda #$00
		sta $0000, X
		sta $0100, X
		sta $0200, X
		sta $0300, X
		sta $0500, X
		sta $0600, X
		sta $0700, X
		lda #$FF ; OAM clear
		sta $0400, X
		inx
	bne :-
	; restore random seed
	pla
	sta prng_seed+1
	pla
	ora #1 ; force at least 1 bit on
	sta prng_seed+0
	; wait for second vblank
	:
		bit $2002
	bpl :-
	;
	; PPU is now warmed up, NES is ready to go!
	;
	jsr load_ramcode
	;
	; detect NTSC/PAL/DENDY
	;
	lda $2002
	lda #%10000000
	sta $2000
	jsr getTVSystem
	cmp #1 ; 1 == PAL
	bne :+
		sta pal
		lda #50
		sta fps
		jmp @system_detected
	:
	cmp #2 ; 2 == DENDY
	bne :+
		sta pal
		lda #50
		sta fps
		jmp @system_detected
	:
	; else presume NTSC
	lda #0
	sta pal
	lda #60
	sta fps
@system_detected:
	lda #0
	sta $2000 ; disable NMI
	; set starting banks and begin
	lda #$00
	sta $5FF8
	lda #$01
	sta $5FF9
	jmp $8009 ; player.s main

;
; NSF
;

.ifdef NSF

.import nsf_ramcode_init
.segment "NSF_INIT"
	pha ; temporarily store track to play
	cpx #1 ; 1 == PAL
	bne :+
		lda #1
		sta pal
		lda #50
		sta fps
		jmp :++
	:
	; else presume NTSC
	lda #0
	sta pal
	lda #60
	sta fps
:
	jsr load_ramcode
	pla
	jmp nsf_ramcode_init

.endif

;
; VECTORS
;

.segment "VECTORS"
.import vec_nmi
.import vec_irq
.word vec_nmi
.word power_on
.word vec_irq

; end of file
